%%
% Author: Florian Schwarzhans
% University: Medical University of Vienna
% 
% Simple Program with graphical User Interface to simulate chemical
% reactions using three different Methods:
% 1 - SSA (Stochastic Simulation Algorithm)
% 2 - CLE (Chemical Langevin Equation)
% 3 - RRE (Reaction Rate Equation)
% 
% This Program is based on the work of D J Higham, which is downloadable
% from      http://www.maths.strath.ac.uk/~aas96106/algfiles.html
% 
% To Use this Program you will need to download and install the String - 
% Toolkit which can be found here
% http://www.mathworks.com/matlabcentral/fileexchange/21710-string-toolkits/content/strings/strsplit.m

function modelling_gui()
%% Initialize all graphical Elements

close all
clear all
f1 = figure('units','normalized','Position',[0.1 0.1 0.8 0.8]);
p0a = uipanel('Parent',f1,'Visible','off','Units','normalized','Position',[0 0 0.3 1]);
p0b = uipanel('Parent',f1,'Units','normalized','Position',[0.3 0 0.7 1]);
axes('Parent',p0b,'Units','normalized','Position',[0.05 0.05 0.9 0.9]);
tx0b = uicontrol('Parent',p0b,'Style','text','Units','normalized','Position',[0.05 0.975 0.9 0.025]);

p00 = uipanel('Parent',f1,'Units','normalized','Position',[0 0 0.3 1]);
pba = uicontrol('Parent',p00,'Style','pushbutton','String','SSA - Stochastic Simulation Algorithm','Callback',@call_ssa,'Units','normalized','Position',[0.1 0.7 0.8 0.2]);
pbb = uicontrol('Parent',p00,'Style','pushbutton','String','CLE - Chemical Langevin Equation','Callback',@call_cle,'Units','normalized','Position',[0.1 0.4 0.8 0.2]);
pbc = uicontrol('Parent',p00,'Style','pushbutton','String','RRE - Reaction Rate Equation','Callback',@call_rre,'Units','normalized','Position',[0.1 0.1 0.8 0.2]);

enstr1 = 'Enter a set of rules and probabilities in the following form: 1+2=3;0.1 (Rule: 1+2=3; Prob: 0.1) and add with [Enter]. ';
enstr2 = 'You can use the Avagadro number (number of molecules in one mole) and the System volume to create dependencies with the variables nA and vol respectively. Example: 1+2=3;1e6/(nA*vol)';
    

p1 = uipanel('Parent',p0a,'Units','normalized','Position',[0.1 0.9 0.8 0.05]);
uicontrol('Parent',p1,'Style','text','String','Please select the number of different species','Units','normalized','Position',[0 0.5 1 0.5]);
sl1 = uicontrol('Parent',p1,'Style','slider','Tag','sl1x','Min',2,'Max',9,'Value',2,'SliderStep',[1/7 1/7],'Units','normalized','Position',[0 0 0.8 0.5],'Callback',@slupdt);
ed1 = uicontrol('Parent',p1,'Style','edit','Tag','sl1','Units','normalized','Position',[0.8 0 0.2 0.5],'String','2');

p2 = uipanel('Parent',p0a,'Units','normalized','Position',[0.1 0.6 0.8 0.25]);
p2a = uipanel('Parent',p2,'Units','normalized','Position',[0 0 0.5 1]);
enstr = uicontrol('Parent',p2a,'Style','edit','Enable','inactive','Max',100,'String','','Units','normalized','Position',[0 0.3 1 0.7]);
ed2 = uicontrol('Parent',p2a,'Style','edit','Tag','ed2x','Units','normalized','Position',[0 0 1 0.3],'Callback',@rlupdt);
listb2 = uicontrol('Parent',p2,'Style','listbox','Tag','ed2','String','','Units','normalized','Position',[0.5 0 0.5 1]);

p3 = uipanel('Parent',p0a,'Units','normalized','Position',[0.1 0.5 0.8 0.05]);
uicontrol('Parent',p3,'Style','text','String','Enter the volume of the System (Default Value = 1e-15)', 'Units','normalized','Position',[0 0 0.7 1]);
ed3 = uicontrol('Parent',p3,'Style','edit','String','1e-15','Units','normalized','Position',[0.7 0 0.3 1]);

txp46 = uicontrol('Parent',p0a,'Style','edit','Enable','inactive','Max',100,'String','Enter the amount of each Species as a concentration. It is automatically multiplied with Avagadro Number and the volume.','Units','normalized','Position',[0.1 0.45 0.8 0.025]);
p46 = uipanel('Parent',p0a,'Units','normalized','Position',[0.1 0.25 0.8 0.2]);
p4 = uipanel('Parent',p46,'Units','normalized','Position',[0 0 0.8 1]);
p4n=[];tx4=[];ed4=[];

p5 = uipanel('Parent',p0a,'Units','normalized','Position',[0.1 0.15 0.8 0.05]);
p5a = uipanel('Parent',p5,'Units','normalized','Position',[0 0.5 1 0.5]);
p5b = uipanel('Parent',p5,'Units','normalized','Position',[0 0 1 0.5]);
uicontrol('Parent',p5a,'Style','text','String','Enter the duration of the reaction','Units','normalized','Position',[0 0 0.8 1]);
uicontrol('Parent',p5b,'Style','text','String','Enter the number of steps ','Units','normalized','Position',[0 0 0.8 1]);
ed5a = uicontrol('Parent',p5a,'Style','edit','String','50','Units','normalized','Position',[0.8 0 0.2 1]);
ed5b = uicontrol('Parent',p5b,'Style','edit','String','250','Units','normalized','Position',[0.8 0 0.2 1]);


p6 = uipanel('Parent',p46,'Units','normalized','Position',[0.8 0 0.2 1]);
cbx6n=[];

subm_ssa = uicontrol('Visible','off','Parent',p0a,'Style','pushbutton','String','submit','Callback',@submt_ssa,'Units','normalized','Position',[0.7 0.05 0.2 0.05]);
subm_cle = uicontrol('Visible','off','Parent',p0a,'Style','pushbutton','String','submit','Callback',@submt_cle,'Units','normalized','Position',[0.7 0.05 0.2 0.05]);
subm_rre = uicontrol('Visible','off','Parent',p0a,'Style','pushbutton','String','submit','Callback',@submt_rre,'Units','normalized','Position',[0.7 0.05 0.2 0.05]);

backbut = uicontrol('Parent',p0a,'Style','pushbutton','String','Back to Main','Callback',@back_main,'Units','normalized','Position',[0.1 0.05 0.2 0.05]);

    function showStarts(cnt)
        for ii=1:cnt
            p4n(ii) = uipanel('Parent',p4,'Units','normalized','Position',[0 (cnt-ii)/cnt 1 1/cnt]);
            tx4(ii) = uicontrol('Parent',p4n(ii),'Style','text','String',['Initial concentration, Species: ' num2str(ii)],'Units','normalized','Position',[0 0 0.7 1]);
            ed4(ii) = uicontrol('Parent',p4n(ii),'Style','edit','Units','normalized','Position',[0.7 0 0.3 1]);
            
            cbx6n(ii) = uicontrol('Parent',p6,'Style','checkbox','String','plot','Units','normalized','Position',[0 (cnt-ii)/cnt 1 1/cnt]);
        end
    end

    
%% Initialize important variables    
rules = [];
rlcount = 0;

V=[];
randn('state',100)
nA = 6.023e23;             % Avagadro's number (number of molecules in one mole)


%% return to main menu
    function back_main(hObject,eventdata)
        modelling_gui();
    end

%% calls the SSA algorithm to simulate the chemical reaction
    function submt_ssa(hObject,eventdata)
        tic

        % get the number of different species
        specnum = (round(get(sl1,'Value')));
        
        vol = eval(get(ed3,'String'));
        X = zeros(specnum,1);
        for ii=1:specnum
            tmp = get(ed4(ii),'String');
            if isempty(tmp)
                X(ii) = 0;
            else
                X(ii) = round(eval(tmp)*nA*vol);
            end
        end
        for ii=1:length(V(1,:))
            if isempty(rules(ii).prob)
                c(ii) = 0;
            else
                c(ii) = eval(rules(ii).prob);
            end
        end

        t=0;    % start value
        tfinal = str2double(get(ed5a,'String'));
        count = 1;
        tvals(1) = 0;   % initial tval = 0
        Xvals(:,1) = X; % initial Xvals are overwritten with initial X



        while t < tfinal   % while t limit has not been reached
            for ii=1:length(c)
                tmp = V(:,ii).*(-1);
                tmp(tmp<0)=0;
                a(ii) = c(ii)*prod(X(find(tmp)));
            end
            asum = sum(a);
            j = min(find(rand<cumsum(a/asum)));
            if ~isempty(j)
                tau = log(1/rand)/asum;    
                X = X + V(:,j);

                count = count + 1;     % increment count (for index)
                t = t + tau;           % save the new t value (t + tau)
                tvals(count) = t;      % write data
                Xvals(:,count) = X;    % write data
            else
                break;
            end
        end
        tt_time = toc;
        set(tx0b,'String',['This simulation took ' num2str(tt_time) ' seconds.']);


        L = length(tvals);
        tnew = zeros(1,2*(L-1));
        tnew(1:2:end-1) = tvals(2:end);
        tnew(2:2:end) = tvals(2:end);
        tnew = [tvals(1),tnew];
        sym = ['+';'o';'*';'.';'x';'s';'d';'p';'h'];
        col = ['r';'g';'b';'c';'m';'y';'k';'k';'k'];
        legst = '';
        for ii=1:specnum
            if get(cbx6n(ii),'Value') == 1
                vals = Xvals(ii,:);
                ynew = zeros(1,2*L-1);
                ynew(1:2:end) = vals;
                ynew(2:2:end-1) = vals(1:end-1);
                plot(tnew,ynew,[col(ii) sym(ii) '-'])
                hold on
                legst = [legst; 'Species ' num2str(ii)];
            end
        end

        xlabel('Time','FontSize',14)
        ylabel('Molecules','FontSize',14)

        legend(legst);
        hold off
        
    end

%% calls the CLE algorithm to simulate the chemical reaction
    function submt_cle(hObject,eventdata)
        
        
        tic
        specnum = (round(get(sl1,'Value')));
        vol = eval(get(ed3,'String'));
        Y = zeros(specnum,1);
        for kk=1:specnum
            tmp = get(ed4(kk),'String');
            if isempty(tmp)
                Y(kk) = 0;
            else
                Y(kk) = round(eval(tmp)*nA*vol);
            end
        end
        
        for ii=1:length(V(1,:))
            if isempty(rules(ii).prob)
                c(ii) = 0;
            else
                c(ii) = eval(rules(ii).prob);
            end
        end
        tfinal = str2double(get(ed5a,'String'));
        L = str2double(get(ed5b,'String'));
        tau = tfinal/L;     % stepsize
        
        Yvals = zeros(specnum,L+1);
        Yvals(:,1) = Y(:,1);

        for k = 1:L                     % for every step
            for ii=1:length(c)
                tmp = V(:,ii).*(-1);
                tmp(tmp<0)=0;
                a(ii) = c(ii)*prod(Y(find(tmp)));
                d(ii) = tau*a(ii) + sqrt(abs(tau*a(ii)))*randn;
                Y = Y + d(ii)*V(:,ii);
            end
        %      Y = Y + d(1)*V(:,1) + d(2)*V(:,2) + d(3)*V(:,3);   % overwrite the initial values 

             Yvals(:,k+1) = Y;  % save values in Yvals
        end
        tt_time = toc;
        set(tx0b,'String',['This simulation took ' num2str(tt_time) ' seconds.']);
        tvals = [0:tau:tfinal];
        
        
        
        sym = ['+';'o';'*';'.';'x';'s';'d';'p';'h'];
        col = ['r';'g';'b';'c';'m';'y';'k';'k';'k'];
        legst = '';
        for ii=1:specnum
            if get(cbx6n(ii),'Value') == 1
                plot(tvals,Yvals(ii,:),[col(ii) sym(ii) '-'])
                hold on
                legst = [legst; 'Species ' num2str(ii)];
            end
        end

        xlabel('Time','FontSize',14)
        ylabel('Molecules','FontSize',14)

        legend(legst);
        hold off
    end

%% calls the RRE algorithm to simulate the chemical reaction
    function submt_rre(hObject,eventdata)
        
        tic
        specnum = (round(get(sl1,'Value')));
        yzero = [];
        
        for kk=1:specnum
            tmp = get(ed4(kk),'String');
            if isempty(tmp)
                tmp = 0;
            else
                tmp = eval(tmp);
            end
            yzero = [yzero;tmp];
        end
        options = odeset('AbsTol',1e-8);
        tspan = [0 str2double(get(ed5a,'String'))];
        
        for ii=1:length(V(1,:))
            if isempty(rules(ii).prob)
                k(ii) = 0;
            else
                k(ii) = eval(rules(ii).prob);
            end
        end

        Vb = V.*(-1);
        Vb(Vb<1) = 0;

        [t,y] = ode15s(@mm_rre,tspan,yzero,options);
        
        tt_time = toc;
        set(tx0b,'String',['This simulation took ' num2str(tt_time) ' seconds.']);
        
        sym = ['+';'o';'*';'.';'x';'s';'d';'p';'h'];
        col = ['r';'g';'b';'c';'m';'y';'k';'k';'k'];
        legst = '';
        for ii=1:specnum
            if get(cbx6n(ii),'Value') == 1
                plot(t,y(:,ii),[col(ii) sym(ii) '-'])
                hold on
                legst = [legst; 'Species ' num2str(ii)];
            end
        end

        xlabel('Time','FontSize',14)
        ylabel('Molecules','FontSize',14)

        legend(legst);
        hold off


%--------------Nested function----------
            function yprime = mm_rre(t,y)
                % MM_RRE    Michaelis-Menten reaction Rate Equation 
                yprime = zeros(specnum,1);
                for iii=1:length(V(:,1))
                    for jj = 1:length(Vb(1,:))
                        yprime(iii) = yprime(iii) + (V(iii,jj)*k(jj)*prod(y(find(Vb(:,jj)))));
                    end
                end
            end
    end

%% Updates the global rule set whenever a new rule has been added
    function rlupdt(hObject,eventdata)
        
        val = strsplit(num2str(get(hObject,'String')),';');
        rlcount = rlcount + 1;
        rules(rlcount).rule = char(val(1));
        rules(rlcount).prob = char(val(2));
        if ~isempty(char(val(1)))
            spl=strsplit(char(val(1)),'=');
            ls = str2double((strsplit(char(spl(1)),'+')));
            rs = str2double((strsplit(char(spl(2)),'+')));
            tmp = zeros(get(sl1,'Value'),1);
            for ii=1:length(ls)
                tmp(ls(ii)) = tmp(ls(ii))-1;    % subtract the left hand side values
            end
            for ii=1:length(rs)
                tmp(rs(ii)) = tmp(rs(ii))+1;    % add the right hand side values
            end  
            V = [V tmp];        % append to matrix
        end
        
        tg = get(hObject,'Tag');
        edt = findobj('Tag',tg(1:end-1));
        strl = '';
        for ii=1:length(rules)
            strl = [strl rules(ii).rule ' --> ' rules(ii).prob];
            if ii<length(rules)
                strl = [strl '|'];
            end
        end
        set(edt,'String',strl);
        set(hObject,'String','');
    end
    
    
%% Updates the Species table according to the current number
    function slupdt(hObject,eventdata)
        val = num2str(round(get(hObject,'Value')));
        tg = get(hObject,'Tag');
        edt = findobj('Tag',tg(1:end-1));
        set(edt,'String',val);
        if exist('p4n')
            for ii=1:length(p4n)
                if ishandle(p4n(ii))
                    delete(tx4(ii));
                    delete(ed4(ii));
                    delete(p4n(ii));
                    delete(cbx6n(ii));
                end
            end
        end
        showStarts(str2double(val));
    end

%% Shows the graphical Objects needed for the SSA Algorithm
    function call_ssa(hObject,eventhandler)
        set(p00,'Visible','off');
        set(p0a,'Visible','on');
        set(p5b,'Visible','off');
        set(subm_ssa,'Visible','on');
        set(enstr,'String',[enstr1 enstr2]);
    end

%% Shows the graphical Objects needed for the CLE Algorithm
    function call_cle(hObject,eventhandler)
        set(p00,'Visible','off');
        set(p0a,'Visible','on');
        set(subm_cle,'Visible','on');
        set(enstr,'String',[enstr1 enstr2]);
    end

%% Shows the graphical Objects needed for the RRE Algorithm
    function call_rre(hObject,eventhandler)
        set(p00,'Visible','off');
        set(p0a,'Visible','on');
        set(p5b,'Visible','off');
        set(p3,'Visible','off');
        set(subm_rre,'Visible','on');
        set(txp46,'Visible','off');
        set(enstr,'String',enstr1);
    end

end
